---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008, 2009 Raul Fechete, Georg Kienesberger   --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Unchecked_Deallocation;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

with Asis;

with Ast2Cfg.Flow_Types; use Ast2Cfg.Flow_Types;
with Ast2Cfg.Nodes; use Ast2Cfg.Nodes;
with Ast2Cfg.Id;
with Ast2Cfg.Declaration_Id;

package Ast2Cfg.CFGs is

   type CFG_Object     is abstract new Flow_Object with private;
   type CFG_Class_Ptr  is access all CFG_Object'Class;

   not overriding
   procedure Init (Cfg:  in out CFG_Object;
                   Name: in Unbounded_String;
                   Pred: in Flow_Class_Ptr;
                   Element: in Asis.Element);

   not overriding
   procedure Free (Cfg: in out CFG_Class_Ptr);

   not overriding
   procedure Free_Special (Cfg: in out CFG_Class_Ptr);

   not overriding
   function Get_Declaration_Id (Cfg: in CFG_Object) return Declaration_Id.Object;

   not overriding
   procedure Set_Head (Cfg: in out CFG_Object; Node: in Node_Class_Ptr);

   not overriding
   function  Get_Head (Cfg: in CFG_Object) return Node_Class_Ptr;

   not overriding
   procedure Set_Tail (Cfg: in out CFG_Object; Node: in Node_Class_Ptr);

   not overriding
   function  Get_Tail (Cfg: in CFG_Object) return Node_Class_Ptr;

   not overriding
   procedure Set_LName (Cfg: in out CFG_Object; Name: in Unbounded_String);

   not overriding
   function  Get_LName (Cfg: in CFG_Object) return String;

   not overriding
   procedure Add_Param (Cfg: in out CFG_Object; Param: in Declaration_Ptr);

   not overriding
   function  Get_Params (Cfg: in CFG_Object) return Declaration_Ptr_List.Object;

   not overriding
   procedure Increment_Node_Sum (Cfg: in out CFG_Object);

   not overriding
   procedure Decrement_Node_Sum (Cfg: in out CFG_Object);

   not overriding
   function Get_Node_Sum (Cfg: in CFG_Object) return Long_Long_Natural;

   type Body_Object is abstract new CFG_Object with private;
   type Spec_Object is abstract new CFG_Object with private;

   -- No specialized init() Methods for the CFG Objects because no extra Paramters are planned
   type Abort_Object  is new Body_Object with private;
   type Block_Object  is new Body_Object with private;
   type Entry_Object  is new Body_Object with private;
   type Except_Object is new Body_Object with private;
   type Func_Object   is new Body_Object with private;
   type Init_Object   is new Body_Object with private;
   type Proc_Object   is new Body_Object with private;
   type Task_Object   is new Body_Object with private;

   type Generic_Object     is new Spec_Object with private;
   type Task_Type_Object   is new Spec_Object with private;
   type Single_Task_Object is new Spec_Object with private;

private

   overriding
   procedure Init (Cfg:  in out CFG_Object;
                   Oid:  in Id.Pointer;
                   Name: in Unbounded_String;
                   Pred: in Flow_Class_Ptr;
                   Element: in Asis.Element);

   type CFG_Object is new Flow_Object with
      record
         Decl_Id: Declaration_Id.Object; -- unique id for tracking the order of declarations
         Head:   Node_Class_Ptr;              -- Pointer to the Head-Node of this CFG
         Tail:   Node_Class_Ptr;              -- Pointer to the Tail-Node of this CFG. may be null!
         LName:  Unbounded_String; -- Link Name - Secondary Name for Call Node <-> CFG Relations
         Params: Declaration_Ptr_List.Object; -- The Parameters of the Subprogram
         Node_Sum: Long_Long_Natural := 0; -- the number of nodes in this CFG, excluding Param_*_Nodes.
      end record;

   type Body_Object is new CFG_Object with
     null record;

   type Spec_Object is new CFG_Object with
     null record;

   type Abort_Object is new Body_Object with
     null record;

   type Block_Object is new Body_Object with
     null record;

   type Entry_Object is new Body_Object with
     null record;

   type Except_Object is new Body_Object with
     null record;

   type Func_Object  is new Body_Object with
     null record;

   type Init_Object  is new Body_Object with
     null record;

   type Proc_Object  is new Body_Object with
     null record;

   type Task_Object  is new Body_Object with
     null record;

   type Generic_Object     is new Spec_Object with
     null record;

   type Task_Type_Object   is new Spec_Object with
     null record;

   type Single_Task_Object is new Spec_Object with
     null record;

end Ast2Cfg.CFGs;
