---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008, 2009 Raul Fechete, Georg Kienesberger   --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Characters.Handling;
with Ada.Exceptions;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

with Asis.Compilation_Units;
with Asis.Exceptions;
with Asis.Errors;
with Asis.Implementation;
with Asis.Elements;
with Asis.Iterator;

with Ast2Cfg.Actuals_For_Traversing;
with Ast2Cfg.Transformation;
with Ast2Cfg.Output; use Ast2Cfg.Output;

package body Ast2Cfg.Context_Processing is

   -----------------------------
   -- Get_Unit_From_File_Name --
   -----------------------------

   function Get_Unit_From_File_Name
     (Ada_File_Name : String;
      The_Context   : Asis.Context)
     return Asis.Compilation_Unit
   is
   begin

      return Asis.Nil_Compilation_Unit;

   end Get_Unit_From_File_Name;

   ------------------------------------
   -- Recursive_Construct_Processing --
   ------------------------------------

   procedure Recursive_Construct_Processing is new
     Asis.Iterator.Traverse_Element
     (State_Information => Actuals_For_Traversing.Traversal_State,
      Pre_Operation     => Actuals_For_Traversing.Pre_Op,
      Post_Operation    => Actuals_For_Traversing.Post_Op);

   ---------------------
   -- Process_Context --
   ---------------------

   procedure Process_Context(The_Context : Asis.Context)
   is
      Units : Asis.Compilation_Unit_List :=
        Asis.Compilation_Units.Compilation_Units (The_Context);

      Next_Unit        : Asis.Compilation_Unit := Asis.Nil_Compilation_Unit;
      Next_Unit_Origin : Asis.Unit_Origins     := Asis.Not_An_Origin;
      Next_Unit_Class  : Asis.Unit_Classes     := Asis.Not_A_Class;

      Unit_Name: Unbounded_String;
   begin

      for J in Units'Range loop
         Next_Unit        := Units (J);
         Next_Unit_Class  := Asis.Compilation_Units.Unit_Class(Next_Unit);
         Next_Unit_Origin := Asis.Compilation_Units.Unit_Origin(Next_Unit);


         Unit_Name := To_Unbounded_String(Ada.Characters.Handling.To_String
                                          (Asis.Compilation_Units.Unit_Full_Name(Next_Unit)));

         case Next_Unit_Class is
            when Asis.A_Public_Declaration |
              Asis.A_Private_Declaration =>

               Unit_Name := Unit_Name & " (spec)";

            when Asis.A_Separate_Body =>

               Unit_Name := Unit_Name & " (subunit)";

            when Asis.A_Public_Body |
              Asis.A_Public_Declaration_And_Body |
              Asis.A_Private_Body =>

               Unit_Name := Unit_Name & " (body)";

            when others =>
               Unit_Name := Unit_Name & " (???)";

         end case;

         case Next_Unit_Origin is
            when Asis.An_Application_Unit =>

               -- Start to process Unit.
               Transformation.Handle_Unit_Started(To_String(Unit_Name), Next_Unit_Class);

               declare
                  Cont_Clause_Elements: constant Asis.Element_List :=
                    Asis.Elements.Context_Clause_Elements (Compilation_Unit => Next_Unit,
                                                           Include_Pragmas  => True);
                  Unit_Decl: Asis.Element := Asis.Elements.Unit_Declaration (Next_Unit);

                  Process_Control: Asis.Traverse_Control := Asis.Continue;
                  Process_State: Actuals_For_Traversing.Traversal_State :=
                    Actuals_For_Traversing.Initial_Traversal_State;
               begin
                  for J in Cont_Clause_Elements'Range loop
                     -- Start to process Element.
                     Process_Control := Asis.Continue;
                     Process_State := Actuals_For_Traversing.Initial_Traversal_State;
                     Recursive_Construct_Processing
                       (Element => Cont_Clause_Elements(J),
                        Control => Process_Control,
                        State   => Process_State);
                     --Stopped to process Element.
                  end loop;

                  -- Start to process Element.
                  Process_Control := Asis.Continue;
                  Process_State := Actuals_For_Traversing.Initial_Traversal_State;
                  Recursive_Construct_Processing
                    (Element => Unit_Decl,
                     Control => Process_Control,
                     State   => Process_State);
                  --Stopped to process Element.
               end;

               Transformation.Handle_Unit_Finished(To_String(Unit_Name), Next_Unit_Class);
               -- Stopped to process Unit.

            when Asis.A_Predefined_Unit =>
               Put(Debug, "[Context_Processing] " &
                   To_String(Unit_Name) &
                   " was skipped as a predefined unit.");

            when Asis.An_Implementation_Unit =>
               Put(Debug, "[Context_Processing] " &
                   To_String(Unit_Name) &
                   " was skipped as an implementation-defined unit.");

            when Asis.Not_An_Origin =>
               Put(Debug, "[Context_Processing] " &
                   To_String(Unit_Name) &
                   " was skipped as nonexistent unit.");
         end case;
         New_Line(Debug);

      end loop;

   exception

      when Ex : Asis.Exceptions.ASIS_Inappropriate_Context  |
        Asis.Exceptions.ASIS_Inappropriate_Container        |
        Asis.Exceptions.ASIS_Inappropriate_Compilation_Unit |
        Asis.Exceptions.ASIS_Inappropriate_Element          |
        Asis.Exceptions.ASIS_Inappropriate_Line             |
        Asis.Exceptions.ASIS_Inappropriate_Line_Number      |
        Asis.Exceptions.ASIS_Failed                         =>

         Put(Error, "[Process_Context] ERROR: ASIS exception (");
         Put(Error, Ada.Exceptions.Exception_Name(Ex));
         Put(Error, ") is raised when processing unit ");
         Put(Error, Ada.Characters.Handling.To_String(Asis.Compilation_Units.Unit_Full_Name(Next_Unit)));
         New_Line(Error);

         Put(Error, "ASIS Error Status is ");
         Put(Error, Asis.Errors.Error_Kinds'Image(Asis.Implementation.Status));
         New_Line(Error);

         Put(Error, "ASIS Diagnosis is ");
         New_Line(Error);
         Put(Error, Ada.Characters.Handling.To_String(Asis.Implementation.Diagnosis));
         New_Line(Error);

         Asis.Implementation.Set_Status;
         raise;

      when Ex : others =>

         Put(Error, "[Process_Context] ERROR: ");
         Put(Error, Ada.Exceptions.Exception_Name(Ex));
         Put(Error, " is raised (");
         Put(Error, Ada.Exceptions.Exception_Information(Ex));
         Put(Error, ")");
         New_Line(Error);
         Put(Error, "when processing unit ");
         Put(Error, Ada.Characters.Handling.To_String(Asis.Compilation_Units.Unit_Full_Name(Next_Unit)));
         New_Line(Error);
         raise;

   end Process_Context;

end Ast2Cfg.Context_Processing;
