---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008, 2009 Raul Fechete, Georg Kienesberger   --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Unchecked_Deallocation;

package body Ast2Cfg.Generic_List is

   function "=" (Left,Right: in Object) return Boolean is
      Left_Cur, Right_Cur: Container_Lists.Cursor;
      use type Ada.Containers.Count_Type;
   begin

      if Container_Lists.Length(Left.Container_List) /=
         Container_Lists.Length(Right.Container_List) then
         return False;
      end if;

      if Container_Lists.Is_Empty(Left.Container_List) and
         Container_Lists.Is_Empty(Right.Container_List) then
         return True;
      end if;

      Left_Cur := Container_Lists.First(Left.Container_List);
      Right_Cur := Container_Lists.First(Right.Container_List);

      while Container_Lists.Has_Element(Left_Cur) loop
         if Container_Lists.Element(Left_Cur) /= Container_Lists.Element(Right_Cur) then
            return False;
         end if;
         Container_Lists.Next(Left_Cur);
         Container_Lists.Next(Right_Cur);
      end loop;

      return True;
   end "=";

   function "=" (Left,Right: in Pointer) return Boolean is
   begin
      if Ptr_Equal(Left, null) and Ptr_Equal(Right, null) then
         return True;
      elsif Ptr_Equal(Left, null) or Ptr_Equal(Right, null) then
         return False;
      end if;

      return "="(Left.all, Right.all);
   end "=";

   -- Initialize list structure information
   procedure Init (Obj: out Object) is
   begin
      Container_Lists.Clear(Obj.Container_List);
      Obj.Curr := Container_Lists.First(Obj.Container_List);
   end Init;

   -- Initialize list structure information
   procedure Init (Ptr: out Pointer) is
   begin
      if Ptr = null then
         Ptr := new Object;
      end if;
      Init(Ptr.all);
   end Init;

   -- Append a new element to the list
   procedure Append (Obj: in out Object; I: in Item) is
   begin
      Container_Lists.Append(Obj.Container_List, I);
   end Append;

   -- Append a new element to the list
   procedure Append (Ptr: in out not null Pointer; I: in Item) is
   begin
      Append(Ptr.all, I);
   end Append;

   procedure Append (Obj: in out Object; O: in out Object) is
      Temp_List: Object := O;
      I: Item;
   begin
      Reset(Temp_List);
      while (Has_Next(Temp_List)) loop
         Get_Next(Temp_List, I);
         Append(Obj, I);
      end loop;
   end Append;

   procedure Append (Ptr: in out not null Pointer; P: in out not null Pointer) is
   begin
      Append(Ptr.all, P.all);
   end Append;

   function Contains (Obj: in Object; I: in Item) return Boolean is
   begin
      return Container_Lists.Contains(Obj.Container_List, I);
   end Contains;

   function Contains (Ptr: in not null Pointer; I: in Item) return Boolean is
   begin
      return Contains (Ptr.all, I);
   end Contains;

   -- Replace the head element of the list
   procedure Replace_Head (Obj: in out Object; I: in Item) is
      Head: Container_Lists.Cursor;
   begin
      Head := Container_Lists.First(Obj.Container_List);
      if Container_Lists.Has_Element(Head) then
         Container_Lists.Replace_Element(Obj.Container_List, Head, I);
      end if;
   end Replace_Head;

   procedure Replace_Head (Ptr: in out not null Pointer; I: in Item) is
   begin
      Replace_Head(Ptr.all, I);
   end Replace_Head;

   -- Insert a new element at the beginning of the list
   procedure Insert_Head (Obj: in out Object; I: in Item) is
   begin
      Container_Lists.Prepend(Obj.Container_List, I);
   end Insert_Head;

   -- Insert a new element at the beginning of the list
   procedure Insert_Head (Ptr: in out not null Pointer; I: in Item) is
   begin
      Insert_Head(Ptr.all, I);
   end Insert_Head;

   function Get_Head (Obj: in Object) return Item is
   begin
      if Container_Lists.Is_Empty(Obj.Container_List) then
         raise List_Exception with "Empty List, no head.";
      end if;

      return Container_Lists.First_Element(Obj.Container_List);
   end Get_Head;

   function Get_Head (Ptr: in not null Pointer) return Item is
   begin
      return Get_Head(Ptr.all);
   end Get_Head;

   function Get_Tail (Obj: in Object) return Item is
   begin
      if Container_Lists.Is_Empty(Obj.Container_List) then
         raise List_Exception with "Empty List, no tail.";
      end if;

      return Container_Lists.Last_Element(Obj.Container_List);
   end Get_Tail;

   function Get_Tail (Ptr: in not null Pointer) return Item is
   begin
      return Get_Tail(Ptr.all);
   end Get_Tail;

   -- Remove the tail element
   procedure Remove_Last (Obj: in out Object) is
      J: Item;
   begin
      Pop(Obj, J);
   end Remove_Last;

   -- Remove the tail element
   procedure Remove_Last (Ptr: in out not null Pointer) is
   begin
      Remove_Last(Ptr.all);
   end Remove_Last;

   -- removes the first element equal to I
   procedure Remove (Obj: in out Object; I: in Item) is
      Pos: Container_Lists.Cursor;
   begin

      Pos := Container_Lists.Find(Obj.Container_List, I);

      if not Container_Lists.Has_Element(Pos) then
         raise List_Exception with "Element to remove not found!";
      end if;

      Container_Lists.Delete(Obj.Container_List, Pos);
   end Remove;

   -- removes the first element equal to I
   procedure Remove (Ptr: in out not null Pointer; I: in Item) is
   begin
      Remove(Ptr.all, I);
   end Remove;

   -- Create a copy of a list
   procedure Duplicate (Src: in out Object; Dst: out Object) is
      New_Obj: Object;
      Cur: Container_Lists.Cursor;
   begin
      Init(New_Obj);

      Cur := Container_Lists.First(Src.Container_List);
      while Container_Lists.Has_Element(Cur) loop
         Container_Lists.Append(New_Obj.Container_List, Container_Lists.Element(Cur));
         Container_Lists.Next(Cur);
      end loop;

      Dst := New_Obj;
   end Duplicate;

   -- Create a copy of a list
   procedure Duplicate (Src: in out not null Pointer; Dst: out Pointer) is
   begin
      if Dst = null then
         Dst := new Object;
         Init(Dst);
      end if;
      Duplicate(Src.all, Dst.all);
   end Duplicate;

   -- Destroy the elements of the list
   procedure Clear (Obj: in out Object) is
   begin
      Container_Lists.Clear(Obj.Container_List);
   end Clear;

   -- Destroy the elements of the list
   procedure Clear (Ptr: in out not null Pointer) is
   begin
      Clear(Ptr.all);
   end Clear;

   procedure Free (Ptr: in out Pointer) is
      procedure Destroy is new Ada.Unchecked_Deallocation(Object, Pointer);
   begin
      if Ptr = null then
         return;
      end if;

      Clear(Ptr.all);
      Destroy(Ptr);
   end Free;

   -- Reset the iterator
   procedure Reset (Obj: in out Object) is
   begin
      Obj.Curr := Container_Lists.First(Obj.Container_List);
   end Reset;

   -- Reset the iterator
   procedure Reset (Ptr: in out not null Pointer) is
   begin
      Reset(Ptr.all);
   end Reset;

   -- Query if the iterator has finished
   function Has_Next (Obj: in Object) return Boolean is
   begin
      return Container_Lists.Has_Element(Obj.Curr);
   end Has_Next;

   -- Query if the iterator has finished
   function Has_Next (Ptr: in not null Pointer) return Boolean is
   begin
      return Has_Next(Ptr.all);
   end Has_Next;

   -- Return the next item from the list
   procedure Get_Next (Obj: in out Object; Next_Item: out Item) is
   begin
      if Container_Lists.Has_Element(Obj.Curr) then
         Next_Item := Container_Lists.Element(Obj.Curr);
         Container_Lists.Next(Obj.Curr);
      else
         raise List_Exception with "List cursor is inconsistent.";
      end if;
   end Get_Next;

   -- Return the next item from the list
   procedure Get_Next (Ptr: in out not null Pointer; Next_Item: out Item) is
   begin
      Get_Next(Ptr.all, Next_Item);
   end Get_Next;

   -- Query the element count
   function Count (Obj: in Object) return Long_Long_Natural is
   begin
      return Long_Long_Natural(Container_Lists.Length(Obj.Container_List));
   end Count;

   -- Query the element count
   function Count (Ptr: in not null Pointer) return Long_Long_Natural is
   begin
      return Count(Ptr.all);
   end Count;

   procedure Push (Obj: in out Object; I: in Item) is
   begin
      Append(Obj, I);
   end Push;

   procedure Push (Ptr: in out not null Pointer; I: in Item) is
   begin
      Push(Ptr.all, I);
   end Push;

   procedure Pop (Obj: in out  Object; I: out Item) is
   begin
      if not Container_Lists.Is_Empty(Obj.Container_List) then
         I := Container_Lists.Last_Element(Obj.Container_List);
         Container_Lists.Delete_Last(Obj.Container_List);
      end if;
   end Pop;

   procedure Pop (Ptr: in out not null Pointer; I: out Item) is
   begin
      Pop(Ptr.all, I);
   end Pop;

   procedure Peek (Obj: in out Object; I: out Item) is
   begin
      if not Container_Lists.Is_Empty(Obj.Container_List) then
         I := Container_Lists.Last_Element(Obj.Container_List);
      end if;
   end Peek;

   procedure Peek (Ptr: in out not null Pointer; I: out Item) is
   begin
      Peek(Ptr.all, I);
   end Peek;

   function Empty (Obj: in Object) return Boolean is
   begin
      return Container_Lists.Is_Empty(Obj.Container_List);
   end Empty;

   function Empty (Ptr: in not null Pointer) return Boolean is
   begin
      return Empty(Ptr.all);
   end Empty;

end Ast2Cfg.Generic_List;
