---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008, 2009 Raul Fechete, Georg Kienesberger   --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Characters.Latin_1;
with Ada.Characters.Handling;
with Ada.Strings.Fixed;
with Ada.Strings.Maps;

package body Ast2Cfg.Utils is

   -- Convert a Wide String to Unbounded_String and Trim it
   function W2u(Str: in Wide_String) return Unbounded_String is
   begin
      return Ada.Strings.Unbounded.To_Unbounded_String(
             Ada.Strings.Fixed.Trim(Ada.Characters.Handling.To_String(Str),
                                    Ada.Strings.Maps.To_Set(" " & Ada.Characters.Latin_1.HT),
                                    Ada.Strings.Maps.To_Set(" " & Ada.Characters.Latin_1.HT)));
   end W2u;

   -- Convert a String to Unbounded_String
   function S2u(Str: in String) return Unbounded_String is
   begin

      -- don't ask. it's because of some esoteric GNAT bug.
      if Str = "" then
         return Ada.Strings.Unbounded.Null_Unbounded_String;
      end if;

      return Ada.Strings.Unbounded.To_Unbounded_String(Str);
   end S2u;

   -- Convert a Unbounded_String to String
   function U2s(U_Str: in Unbounded_String) return String is
   begin
      return Ada.Strings.Unbounded.To_String(U_Str);
   end U2s;

   -- Convert a Wide String to String and Trim it
   function W2s(Str: in Wide_String) return String is
   begin
      return
        Ada.Strings.Fixed.Trim(Ada.Characters.Handling.To_String(Str),
                               Ada.Strings.Maps.To_Set(" " & Ada.Characters.Latin_1.HT),
                               Ada.Strings.Maps.To_Set(" " & Ada.Characters.Latin_1.HT));
   end W2s;

   function Mask(Str: in String) return String is
   begin
      return U2s(Mask(S2u(Str)));
   end Mask;

   function Mask(U_Str: in Unbounded_String) return Unbounded_String is
      I: Integer := 1;
      Retval: Unbounded_String := U_Str;
   begin
      while I <= Length(Retval) loop
         if Element(Retval, I) = '"' then
            Replace_Slice(Retval, I, I, "\""");
            I := I + 1;
         elsif Element(Retval, I) = '\' then
            Replace_Slice(Retval, I, I, "\\");
            I := I + 1;
         elsif Element(Retval, I) = Ada.Characters.Latin_1.LF then
            Replace_Slice(Retval, I, I, "\n");
            I := I + 1;
         elsif Element(Retval, I) = '<' then
            Replace_Slice(Retval, I, I, "\<");
            I := I + 1;
         elsif Element(Retval, I) = '>' then
            Replace_Slice(Retval, I, I, "\>");
            I := I + 1;
         end if;
         I := I + 1;
      end loop;
      return Retval;
   end Mask;

   function Tokenize(Str: in String; Token: in String)
                    return UStr_List.Object is
      SubStr     : Unbounded_String  := S2u(Str);
      Name       : Unbounded_String  := S2u("");
      Next_Index : Natural           := 0;
      Token_List : UStr_List.Object;
   begin
      while U2s(SubStr)'Last > 0 loop
         Next_Index := Index(SubStr, Token);
         if Next_Index > 0 then
            Name := S2u(U2s(SubStr)(1..Next_Index-1));
            SubStr := S2u(U2s(SubStr)(Next_Index+1..U2s(SubStr)'Last));
         else
            Name := SubStr;
            SubStr := S2u("");
         end if;
         UStr_List.Append(Token_List, Name);
      end loop;
      return Token_List;
   end Tokenize;

   --
   -- Create one single String out of an Unbounded String List
   --
   function Merge_UStr_List(UString_List: in UStr_List.Object; Separator: in String) return String is
   begin
      return U2s(Merge_UStr_List(UString_List, Separator));
   end Merge_UStr_List;

   --
   -- Create one single Unbounded_String out of an Unbounded String List
   --
   function Merge_UStr_List(UString_List: in UStr_List.Object; Separator: in String) return Unbounded_String is
      List:         UStr_List.Object := UString_List;
      Output, Item: Unbounded_String := S2u("");
      First:        Boolean          := True;
   begin
      UStr_List.Reset(List);
      -- Walk the List
      while UStr_List.Has_Next(List) loop
         UStr_List.Get_Next(List, Item);
         if not First then
            Output := Output & Separator;
         else
            First := False;
         end if;
         Output := Output & Item;
      end loop;
      return Output;
   end Merge_UStr_List;

   --
   -- Create one single String out of a List of Unbounded String Lists
   --
   function Merge_UStr_List_List (UString_List_List: in UStr_List_List.Object;
                                  Separator1: in String; Separator2: in String) return String is
      List:      UStr_List.Object;
      List_List: UStr_List_List.Object := UString_List_List;
      Output:    Unbounded_String      := S2u("");
      First:     Boolean               := True;
   begin
      UStr_List_List.Reset(List_List);
      while UStr_List_List.Has_Next(List_List) loop
         UStr_List_List.Get_Next(List_List, List);
         if not First then
            Output := Output & Separator2;
         else
            First := False;
         end if;
         Output := Output & Unbounded_String'(Merge_UStr_List(List, Separator1));
      end loop;
      return To_String(Output);
   end Merge_UStr_List_List;

   function UStr_List_Dump (List: in UStr_List.Object) return String is
      MyList: UStr_List.Object := List;
      Output, Component_Name: Unbounded_String := S2u("");
   begin
      UStr_List.Reset(MyList);
      while UStr_List.Has_Next(MyList) loop
         UStr_List.Get_Next(MyList, Component_Name);
         if Output /= S2u("") then
            Output := Output & " ";
         end if;
         Output := Output & Component_Name;
      end loop;
      return U2s(Output);
   end UStr_List_Dump;

end Ast2Cfg.Utils;
